@rem = ' This is a perl script for NT and UNIX.
@echo off
goto endofperl
@rem ';
# -*- tab-width: 4 -*-
#---------------------------------------------------------------------------+
# Licensed Materials - Property of IBM
# Rational ClearCase
# Copyright IBM Corp. 2000, 2005. All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract
# with IBM Corp.
#---------------------------------------------------------------------------+

###########################################################################
# flush_syncmgr_cache.bat
#
# This perl script is intended to be used only by MultiSite customers 
# for setting up their MultiSite servers in a cluster using a 3rd party 
# High Availability solution. 
###########################################################################

use strict;
use Getopt::Long;

my $thisscript = 'flush_syncmgr_cache';
my $tracing = ($ENV{TRACE_SUBSYS} && ($ENV{TRACE_SUBSYS} =~ m/$thisscript/));
my $verboseness = ($ENV{TRACE_VERBOSITY});
my $trace_msg ='';
my $error_msg = '';
my $logdir;
my $producthome;
my $error_logname = '';

use Config;

$CFG::including_script = $thisscript;

my $arch = $^O ? $^O : $main::Config{archname};
$CFG::NT = (($arch =~ /^MSWin32/) || ($arch eq 'i386-win32')) ? 1 : 0;

# Determine the correct path for including syncmgr_common.pl
# CLEARCASEHOME e.v. should always be set on NT when called by ccperlpp.pl,
# so exit if it's not set. On UNIX, assume /opt/rational/clearcase if user
# hasn't explicitly set CLEARCASEHOME (or deprecated ATRIAHOME).
#
if ($CFG::NT) {
    #  It's NT
    if (!exists $ENV{CLEARCASEHOME} ) {
	print STDOUT "CLEARCASEHOME environment variable not set\n";
	exit 1;
    }
    $producthome = $ENV{CLEARCASEHOME};
    chdir $producthome;
    #  For CC V3.2.1 only, uncomment the following line:
    #push(@INC, "$producthome\\config\\cron");
    #  For CC V4.0 and above, uncomment the following line:
    push(@INC, "$producthome\\config\\scheduler\\tasks");
    $logdir = "$producthome\\var\\log\\flush_cache_logs";
} else {
    #  It's UNIX
    # Check that root is running the program; die otherwise
    my $user = getpwuid ($>);
    if ( $user ne "root" ) {
    	print STDOUT "Must be root to execute flush_syncmgr_cache script\n";
	exit 1;
    }

    if (!exists $ENV{CLEARCASEHOME}) {
        # check for (now deprecated) ATRIAHOME e.v.; use if set.
        if (exists $ENV{ATRIAHOME} ) {
            $ENV{CLEARCASEHOME} = $ENV{ATRIAHOME};
        } else {
            $ENV{CLEARCASEHOME} = "/opt/rational/clearcase";
        }
    }
    $producthome = $ENV{CLEARCASEHOME};

    chdir $producthome;
    #  For CC V3.2.1 only, uncomment the following line:  
    #push(@INC, "$producthome/config/multisite");
    #  For CC V4.0 and above, uncomment the following line:
    push(@INC, "$producthome/config/scheduler/tasks");
    $logdir = "/var/adm/rational/clearcase/log/flush_cache_logs";
}

# set script version to test for mismatch against the syncmgr_common.pl version.
$CFG::MS_script_version = "7.0.0.0_syncmgr";

require 'syncmgr_common.pl';
require 5.8.6;

if (!$CFG::MScommon_version) {
    $CFG::MScommon_version = "(unknown older version)";
}

# check for mismatched versions - syncmgr_common.pl should set
# $CFG::MScommon_version; if not set or unequal, may indicate that an
# older/newer version is being included improperly.

if ($CFG::MScommon_version ne $CFG::MS_script_version) {
    die "Version mismatch between require file 'syncmgr_common.pl' " .
        "(version: ${CFG::MScommon_version}) and \n" .
        "the MultiSite script which includes it, 'flush_syncmgr_cache.bat' " .
        "(version: ${CFG::MS_script_version}).\n" . 
        "Ensure that old versions of the script/require file are not in use.\n";
}

$CFG::failcount = 0;

if ($tracing) {
    $| = 1 ;  # do not buffer stdout
}	

  traceprint ("running on architecture $arch\n", 4);

###############################################################################
# Main 
#

# Parse the command line options:

    traceprint ("ARGV: " . join(' ',@ARGV) . "\n", 1);

my $VOBtag = '';
my $help = 0;
my $all_local = 0;
my $VOBs;
my $listfile = '';
my $usage = 
    "$thisscript  [-h] [-all | -vobs <VOB-tag, ...> | list_file ]\n" .
    "    More help is available by using the '-help' command-line switch.\n";
my $helpmsg = <<ENDHELP;
================================================================================

"flush_syncmgr_cache" is used to flush the cache maintained by the
syncmgr_server. This script is used to setup ClearCase servers for high
availability scenarios. 


$usage
Command line options:
    -help               Prints this message and exits.
    
    -all                Check all local replicated VOBs. Other "-vob"
                        parameters on command line and any listfile specified
                        will be ignored if '-all' is specified.

    -vobs               Full VOB-tag,  in comma-separated list. No whitespace.

    list_file           Path to file containing a set of directives to 
                        execute. Must be the final parameter specified on
                        the command line. (Default is no list-file).

                        List-file may contain the following directive:
                        vob:<VOBtag1> 
                        vob:<VOBtag2>
			... 
                        vob:<VOBtagN> 
 			One entry to a line, no whitespace

================================================================================
ENDHELP

my $st = GetOptions("help" => \$help,
                "all" => \$all_local,
		"vobs=s" => \$VOBs);

if ($help) {
    print STDOUT "$helpmsg\n";
    exit 0;
}

if (!($all_local) && !($VOBs) && (scalar @ARGV == 0)) {
    print STDOUT "$usage\n";
    exit 0;
}

#
# Validate that the log directory actually exists and that we can write to it.
#

my $log_status = CheckLogDir($logdir);

if ($log_status != 0) { 

    if($log_status == 1){
    	$error_msg = "ERROR: Unable to create log directory $logdir\n";
    }elsif ($log_status == 2){
    	$error_msg = "ERROR: Log directory not writeable: $logdir\n";
    }else {
    	$error_msg = "ERROR: Unknown error checking $logdir\n";
    }

    print STDOUT $error_msg;
    exit 1;
}
		

#
# Redirect STDERR to a uniquely named log file
#

$error_logname = RedirectSTDERRtoLog ($logdir, 'flush_cache');

if (scalar @ARGV > 1) {
    $error_msg = "ERROR: too many list-files specified in cmd line!\n";
    WriteLog ($error_logname, $error_msg);
    LogError ($thisscript, $error_logname);
    print STDOUT $error_msg;
    exit 1;
}

if (!$st) {  # bad status or more than one arg left
    $error_msg = "ERROR: error getting command line options.\nUsage:\n$usage\n";
    WriteLog ($error_logname, $error_msg);
    LogError ($thisscript, $error_logname);
    print STDOUT $error_msg;
    exit 1;
}

if (scalar @ARGV) {  # list-file specified
    $listfile = $ARGV[0];
    chomp $listfile;
} 

if ( $tracing) {
    $trace_msg = "\n===============================================".
    	"=============\n" .
    	"Beginning execution of MultiSite script '$thisscript' at " .
    	MakeTimestamp() . ".\n\n".
    	"command options specified or defaulted:\n  logdir: $logdir\n".	
    	"  all: $all_local\n    vobs: $VOBs\n".
	"  list: $listfile\n\n".
	"error log filename: $error_logname\n";

	WriteLog ("$trace_msg\n");
        $CFG::failcount++;
}
traceprint ($trace_msg, 1);

#
# Get the leaf name(s) of the VOB(s).

my @vobs;
my $tag = '';
my $res = 0;

#
# if -all specified, ignore -vob and listfile

if ($all_local) {
   my $cmd = "$CFG::MultiTool syncserver -inquire -all";
   my $syncserver_output = DoCmd($cmd);
   my @array = split /\n/,$syncserver_output;
   chomp @array;
   my $line; 
   foreach $line (@array) {	
      chomp $line;
      if ($line =~ /.*vob:(.*) sync method: (.*)/) {
         my $vob = $1;
         my $mode = $2;
         traceprint ("vob = $vob mode - $mode\n", 2);
         if ($mode eq "managed") {
             my $res = DoCmd("$CFG::MultiTool syncserver -nmanage -vob $vob");
             my $res = DoCmd("$CFG::MultiTool syncserver -manage -vob $vob");
         }
      }
   }
} 

#
# If -vobs, separate list into an array of VOB-tags. Ignore listfile

if ($VOBs) {
    @vobs  = split /,/,$VOBs;
    chomp @vobs;
    $listfile = '';
}


#
# If listfile specified, make sure it exists

if ($listfile && !(-e $listfile )) {
    $error_msg = "ERROR: List file $listfile does not exist\n";
    WriteLog ($error_logname, $error_msg);
    LogError ($thisscript, $error_logname);
    print STDOUT $error_msg;
    exit 1;
}

#
# It exists. Try to read the list of VOB-tags

if ($listfile) {
    my ($option, $val);
    traceprint ("opening list file '$listfile'.\n", 2);
    if ( open (SCR, "<$listfile")){
    	while (<SCR>) {
            chomp;
            ($option, $val) = split(":", $_);
            traceprint ("listfile - option: $option value: $val\n",2);
            if ("vob" =~ m/^$option/) {

        	@vobs = ($val, @vobs);
            	traceprint ("found vob...$val\n",2);

	    } else {

		$error_msg = "ERROR: unrecognized option '$option' in listfile $listfile\n";
		WriteLog($error_logname, $error_msg);
    		LogError ($thisscript, $error_logname);
		print STDOUT $error_msg;
        	$CFG::failcount++;
            }
    	}
    } else {
	# Couldn't open the list file. Output error and exit.
	$error_msg = "ERROR: Could not open $listfile for reading\n";
	WriteLog ($error_logname, $error_msg);
    	LogError ($thisscript, $error_logname);
    	print STDOUT $error_msg;
    	exit 1;
    }
	
}



 LOOP: foreach $tag (@vobs) {
     my $line = DoCmd("$CFG::MultiTool syncserver -inquire -vob $tag");
     chomp $line;
     if ($line =~ /.*sync method: (.*)/) {
         my $mode = $1;
         traceprint ("vob = $tag mode - $mode\n", 2);
         if ($mode eq "managed") {
             $res = DoCmd("$CFG::MultiTool syncserver -nmanage -vob $tag");
             $res = DoCmd("$CFG::MultiTool syncserver -manage -vob $tag");
         }
     }
     
     next LOOP;
 }

my $msg = "Successfully completed $thisscript script\n";
if ($tracing) {
	WriteLog ($error_logname, $msg);
        $CFG::failcount++;
}
traceprint ($msg, 1);

print STDOUT "";

# Remove empty logfile if no errors were encountered:

close STDERR;
if (!$CFG::failcount) {
	unlink ($error_logname) || warn "Unable to delete $error_logname\n";
}

close STDOUT;

__END__
:endofperl
ccperl -S ccperlpp.pl %0.bat %*
